@@ -245,7 +245,7 @@ bool KIO::DesktopExecParser::hasSchemeHandler(const QUrl &url)
         return true;
     }
     if (KProtocolInfo::isKnownProtocol(url)) {
-        return false; // see KRun::init()... this is case B, we prefer kioslaves over the competition
+        return false; // this is case B, we prefer kioslaves over the competition
     }
     const KService::Ptr service = KMimeTypeTrader::self()->preferredService(QLatin1String("x-scheme-handler/") + url.scheme());
     if (service) {
@@ -298,7 +298,7 @@ QStringList KIO::DesktopExecParser::resultingArguments() const
 {
     QString exec = d->service.exec();
     if (exec.isEmpty()) {
-        qCWarning(KIO_CORE) << "KRun: no Exec field in `" << d->service.entryPath() << "' !";
+        qCWarning(KIO_CORE) << "No Exec field in `" << d->service.entryPath() << "' !";
         return QStringList();
     }
 
@@ -309,7 +309,7 @@ QStringList KIO::DesktopExecParser::resultingArguments() const
     KRunMX2 mx2(d->urls);
 
     if (!mx1.expandMacrosShellQuote(exec)) {    // Error in shell syntax
-        qCWarning(KIO_CORE) << "KRun: syntax error in command" << d->service.exec() << ", service" << d->service.name();
+        qCWarning(KIO_CORE) << "Syntax error in command" << d->service.exec() << ", service" << d->service.name();
         return QStringList();
     }
 
@@ -327,6 +327,12 @@ QStringList KIO::DesktopExecParser::resultingArguments() const
         return result;
     }
 
+    // Return true for non-KIO desktop files with explicit X-KDE-Protocols list, like vlc, for the special case below
+    auto isNonKIO = [this]() {
+        const QStringList protocols = d->service.property(QStringLiteral("X-KDE-Protocols")).toStringList();
+        return !protocols.isEmpty() && !protocols.contains(QLatin1String("KIO"));
+    };
+
     // Check if we need kioexec, or KIOFuse
     bool useKioexec = false;
     org::kde::KIOFuse::VFS kiofuse_iface(QStringLiteral("org.kde.KIOFuse"),
@@ -335,43 +341,30 @@ QStringList KIO::DesktopExecParser::resultingArguments() const
     struct MountRequest { QDBusPendingReply<QString> reply; int urlIndex; };
     QVector<MountRequest> requests;
     requests.reserve(d->urls.count());
-    QStringList appSupportedProtocols = supportedProtocols(d->service);
-    if (!mx1.hasUrls) {
-        for (int i = 0; i < d->urls.count(); ++i)  {
-            const QUrl url = d->urls.at(i);
-            if (!url.isLocalFile() && !hasSchemeHandler(url)) {
-                // Lets try a KIOFuse mount instead.
-                requests.push_back({ kiofuse_iface.mountUrl(url.toString()), i });
-            }
+    const QStringList appSupportedProtocols = supportedProtocols(d->service);
+    for (int i = 0; i < d->urls.count(); ++i)  {
+        const QUrl url = d->urls.at(i);
+        const bool supported = mx1.hasUrls ? isProtocolInSupportedList(url, appSupportedProtocols) : url.isLocalFile();
+        if (!supported) {
+            // if FUSE fails, we'll have to fallback to kioexec
+            useKioexec = true;
         }
-    } else if (!appSupportedProtocols.contains(QLatin1String("KIO"))) {
-        for (int i = 0; i < d->urls.count(); ++i)  {
-            const QUrl url = d->urls.at(i);
-            const bool supported = isProtocolInSupportedList(url, appSupportedProtocols);
-            // NOTE: Some non-KIO apps may support the URLs (e.g. VLC supports smb://)
-            // but will not have the password if they are not in the URL itself.
-            // Hence convert URL to KIOFuse equivalent in case there is a password.
-            // @see https://pointieststick.com/2018/01/17/videos-on-samba-shares/
-            // @see https://bugs.kde.org/show_bug.cgi?id=330192
-            if (!supported || (!url.userName().isEmpty() && url.password().isEmpty())) {
-                requests.push_back({ kiofuse_iface.mountUrl(url.toString()), i });
-            }
+        // NOTE: Some non-KIO apps may support the URLs (e.g. VLC supports smb://)
+        // but will not have the password if they are not in the URL itself.
+        // Hence convert URL to KIOFuse equivalent in case there is a password.
+        // @see https://pointieststick.com/2018/01/17/videos-on-samba-shares/
+        // @see https://bugs.kde.org/show_bug.cgi?id=330192
+        if (!supported || (!url.userName().isEmpty() && url.password().isEmpty() && isNonKIO())) {
+            requests.push_back({ kiofuse_iface.mountUrl(url.toString()), i });
         }
     }
 
-    // Doesn't matter that we've blocked here to process the replies.
-    // Main thing that we want is to send the mount requests without blocking.
     for (auto &request : requests) {
         request.reply.waitForFinished();
-        if (request.reply.isError()) {
-            useKioexec = true;
-            // At this point we should just send the original urls to kioexec.
-            // There's no point sending urls to kioexec that go through kiofuse.
-            break;
-        }
     }
+    const bool fuseError = std::any_of(requests.cbegin(), requests.cend(), [](const MountRequest &request) { return request.reply.isError(); });
 
-    if (useKioexec) {
+    if (fuseError && useKioexec) {
         // We need to run the app through kioexec
         result << kioexecPath();
         if (d->tempFiles) {
@@ -389,7 +382,9 @@ QStringList KIO::DesktopExecParser::resultingArguments() const
     // At this point we know we're not using kioexec, so feel free to replace
     // KIO URLs with their KIOFuse local path.
     for (const auto &request : qAsConst(requests)) {
-        d->urls[request.urlIndex] = QUrl::fromLocalFile(request.reply.value());
+        if (!request.reply.isError()) {
+            d->urls[request.urlIndex] = QUrl::fromLocalFile(request.reply.value());
+        }
     }
 
     if (appHasTempFileOption) {
@@ -437,7 +432,7 @@ QStringList KIO::DesktopExecParser::resultingArguments() const
         }
         terminal += QLatin1Char(' ') + d->service.terminalOptions();
         if (!mx1.expandMacrosShellQuote(terminal)) {
-            qCWarning(KIO_CORE) << "KRun: syntax error in command" << terminal << ", service" << d->service.name();
+            qCWarning(KIO_CORE) << "Syntax error in command" << terminal << ", service" << d->service.name();
             return QStringList();
         }
         mx2.expandMacrosShellQuote(terminal);
@@ -448,14 +443,17 @@ QStringList KIO::DesktopExecParser::resultingArguments() const
     KShell::Errors err;
     QStringList execlist = KShell::splitArgs(exec, KShell::AbortOnMeta | KShell::TildeExpand, &err);
     if (err == KShell::NoError && !execlist.isEmpty()) { // mx1 checked for syntax errors already
-        // Resolve the executable to ensure that helpers in libexec are found.
-        // Too bad for commands that need a shell - they must reside in $PATH.
-        QString exePath = QStandardPaths::findExecutable(execlist.first());
-        if (exePath.isEmpty()) {
-            exePath = QFile::decodeName(CMAKE_INSTALL_FULL_LIBEXECDIR_KF5 "/") + execlist.first();
-        }
-        if (QFile::exists(exePath)) {
-            execlist[0] = exePath;
+        const QString executable = execlist.at(0);
+        if (QDir::isRelativePath(executable)) {
+            // Resolve the executable to ensure that helpers in libexec are found.
+            // Too bad for commands that need a shell - they must reside in $PATH.
+            QString exePath = QStandardPaths::findExecutable(executable);
+            if (exePath.isEmpty()) {
+                exePath = QFile::decodeName(CMAKE_INSTALL_FULL_LIBEXECDIR_KF5 "/") + executable;
+            }
+            if (QFile::exists(exePath)) {
+                execlist[0] = exePath;
+            }
         }
     }
     if (d->service.substituteUid()) {
